function [r1_00,eta]=auto_gum(estimfull,test_func,y,z,x,namey,namexos,namecomp,prests,boundsvar,dropna,eta,m2prt_test,descent,ncomp,varargin)
    
// PURPOSE: in an automatic estimation, estimates the gum 
// (Gerneralized Unrestricted Model)
// ------------------------------------------------------------
// INPUT:
// * estim_func = the function performing estimation
// * test-func = the function performing specification tests
// * y = vector of the endogenous variable
// * x = matrix of the exogenous variables
// * z = matrix of the compulsory variables (the ones that must
//   be in the regression whatever significance they have
// * namexos = a string vector containing the names of the
//   exoegnous variables
// * rmod = a predefined tlist result where estimation and tests
//   results will be stored
// * eta = a vector equal to the significance levels for the
//   specification tests
// * m2prt_test = the string vector collecting the names of the
//   specification tests
// * descent = the factor by which p-values of specifiaction
//   tests muts be divided to obtain the new levels used in the
//   subsequent estimations
// * varargin = any optional argument to function estim_func
// ------------------------------------------------------------
// OUPTUT:
// * r1_00 = results structure of the gum model 
// * eta = the -if necessary adjusted- critical levels of 
//   specification tests
// ------------------------------------------------------------
// NOTES:
// * used by automatic1()
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002-2005
// http://dubois.ensae.net/grocer.html
 
    
// estimate the initial model
r1_00=estimfull(y,x,z,namey,namexos,namecomp,prests,boundsvar,dropna,ncomp,[1:size(x,2)+size(z,2)],varargin(:))
[val,p]=test_func(r1_00)
r1_00(1)($+1)='spec_test'
r1_00('spec_test')=[val p]
r1_00(1)($+1)='m_test'
r1_00('m_test')=m2prt_test

// if needed adjust the size of eta to the one of p
if ~isempty(eta) then
   if size(eta,'*') == 1 then
      eta=eta+0*p
   end
  
   for i=1:size(p,1)
      if p(i) < eta(i) then
    // must adjust the size level of the corresponding
    // specification test
         if p(i) > %eps*descent then
            eta(i)=p(i)/descent
         else
    // the adjustment would lead to a level lower than
    // scilab precision ; so set it to
            eta(i)=0
         end
         disp(' ')
         chars='!!!!!!!!!!'
         disp(chars+chars+chars+chars+chars+chars)
         warning('level of specification test '+ m2prt_test(i+1)+' had ...
              to be ajusted to:'+string(eta(i)))
         disp('a) this can be the sign of a missing variable in your model')
         disp('b) it could be worthwhile to check what happens if you run the ')
         disp('programm without the corresponding specification test')
         disp(chars+chars+chars+chars+chars+chars)
         disp(' ')
      end
   end
end

endfunction
